# -- coding: utf-8 --
# Created by LoginRadius Development Team
# Copyright 2019 LoginRadius Inc. All rights reserved.
#


class MultiFactorAuthenticationApi:

    def __init__(self, lr_object):
        """
        :param lr_object: this is the reference to the parent LoginRadius object.
        """
        self._lr_object = lr_object

    def mfa_configure_by_access_token(self, access_token, sms_template2_f_a=None):
        """This API is used to configure the Multi-factor authentication after login by using the access token when MFA is set as optional on the LoginRadius site.
        
        Args:
            access_token: Uniquely generated identifier key by LoginRadius that is activated after successful authentication.
            sms_template2_f_a: SMS Template Name
		
        Returns:
            Response containing Definition of Complete Multi-Factor Authentication Settings data
        5.7
        """

        if(self._lr_object.is_null_or_whitespace(access_token)):
            raise Exception(self._lr_object.get_validation_message("access_token"))

        query_parameters = {}
        query_parameters["access_token"] = access_token
        query_parameters["apiKey"] = self._lr_object.get_api_key()
        if(not self._lr_object.is_null_or_whitespace(sms_template2_f_a)):
            query_parameters["smsTemplate2FA"] = sms_template2_f_a

        resource_path = "identity/v2/auth/account/2fa"
        return self._lr_object.execute("GET", resource_path, query_parameters, {})

    def mfa_update_setting(self, access_token, multi_factor_auth_model_with_lockout, fields=''):
        """This API is used to trigger the Multi-factor authentication settings after login for secure actions
        
        Args:
            access_token: Uniquely generated identifier key by LoginRadius that is activated after successful authentication.
            multi_factor_auth_model_with_lockout: Model Class containing Definition of payload for MultiFactorAuthModel With Lockout API
            fields: The fields parameter filters the API response so that the response only includes a specific set of fields
		
        Returns:
            Response containing Definition for Complete profile data
        5.9
        """

        if(self._lr_object.is_null_or_whitespace(access_token)):
            raise Exception(self._lr_object.get_validation_message("access_token"))
        if(multi_factor_auth_model_with_lockout is None):
            raise Exception(self._lr_object.get_validation_message("multi_factor_auth_model_with_lockout"))

        query_parameters = {}
        query_parameters["access_token"] = access_token
        query_parameters["apiKey"] = self._lr_object.get_api_key()
        if(not self._lr_object.is_null_or_whitespace(fields)):
            query_parameters["fields"] = fields

        resource_path = "identity/v2/auth/account/2fa/verification/otp"
        return self._lr_object.execute("PUT", resource_path, query_parameters, multi_factor_auth_model_with_lockout)

    def mfa_update_by_access_token(self, access_token, multi_factor_auth_model_by_google_authenticator_code, fields='',
        sms_template=None):
        """This API is used to Enable Multi-factor authentication by access token on user login
        
        Args:
            access_token: Uniquely generated identifier key by LoginRadius that is activated after successful authentication.
            multi_factor_auth_model_by_google_authenticator_code: Model Class containing Definition of payload for MultiFactorAuthModel By GoogleAuthenticator Code API
            fields: The fields parameter filters the API response so that the response only includes a specific set of fields
            sms_template: SMS Template name
		
        Returns:
            Response containing Definition for Complete profile data
        5.10
        """

        if(self._lr_object.is_null_or_whitespace(access_token)):
            raise Exception(self._lr_object.get_validation_message("access_token"))
        if(multi_factor_auth_model_by_google_authenticator_code is None):
            raise Exception(self._lr_object.get_validation_message("multi_factor_auth_model_by_google_authenticator_code"))

        query_parameters = {}
        query_parameters["access_token"] = access_token
        query_parameters["apiKey"] = self._lr_object.get_api_key()
        if(not self._lr_object.is_null_or_whitespace(fields)):
            query_parameters["fields"] = fields
        if(not self._lr_object.is_null_or_whitespace(sms_template)):
            query_parameters["smsTemplate"] = sms_template

        resource_path = "identity/v2/auth/account/2fa/verification/googleauthenticatorcode"
        return self._lr_object.execute("PUT", resource_path, query_parameters, multi_factor_auth_model_by_google_authenticator_code)

    def mfa_update_phone_number_by_token(self, access_token, phone_no2_f_a, sms_template2_f_a=None):
        """This API is used to update the Multi-factor authentication phone number by sending the verification OTP to the provided phone number
        
        Args:
            access_token: Uniquely generated identifier key by LoginRadius that is activated after successful authentication.
            phone_no2_f_a: Phone Number For 2FA
            sms_template2_f_a: SMS Template Name
		
        Returns:
            Response containing Definition for Complete SMS data
        5.11
        """

        if(self._lr_object.is_null_or_whitespace(access_token)):
            raise Exception(self._lr_object.get_validation_message("access_token"))

        if(self._lr_object.is_null_or_whitespace(phone_no2_f_a)):
            raise Exception(self._lr_object.get_validation_message("phone_no2_f_a"))

        query_parameters = {}
        query_parameters["access_token"] = access_token
        query_parameters["apiKey"] = self._lr_object.get_api_key()
        if(not self._lr_object.is_null_or_whitespace(sms_template2_f_a)):
            query_parameters["smsTemplate2FA"] = sms_template2_f_a

        body_parameters = {}
        body_parameters["phoneNo2FA"] = phone_no2_f_a

        resource_path = "identity/v2/auth/account/2fa"
        return self._lr_object.execute("PUT", resource_path, query_parameters, body_parameters)

    def mfa_reset_google_auth_by_token(self, access_token, googleauthenticator):
        """This API Resets the Google Authenticator configurations on a given account via the access token
        
        Args:
            access_token: Uniquely generated identifier key by LoginRadius that is activated after successful authentication.
            googleauthenticator: boolean type value,Enable google Authenticator Code.
		
        Returns:
            Response containing Definition of Delete Request
        5.12.1
        """

        if(self._lr_object.is_null_or_whitespace(access_token)):
            raise Exception(self._lr_object.get_validation_message("access_token"))

        query_parameters = {}
        query_parameters["access_token"] = access_token
        query_parameters["apiKey"] = self._lr_object.get_api_key()

        body_parameters = {}
        body_parameters["googleauthenticator"] = googleauthenticator

        resource_path = "identity/v2/auth/account/2fa/authenticator"
        return self._lr_object.execute("DELETE", resource_path, query_parameters, body_parameters)

    def mfa_reset_sms_auth_by_token(self, access_token, otpauthenticator):
        """This API resets the SMS Authenticator configurations on a given account via the access token.
        
        Args:
            access_token: Uniquely generated identifier key by LoginRadius that is activated after successful authentication.
            otpauthenticator: Pass 'otpauthenticator' to remove SMS Authenticator
		
        Returns:
            Response containing Definition of Delete Request
        5.12.2
        """

        if(self._lr_object.is_null_or_whitespace(access_token)):
            raise Exception(self._lr_object.get_validation_message("access_token"))

        query_parameters = {}
        query_parameters["access_token"] = access_token
        query_parameters["apiKey"] = self._lr_object.get_api_key()

        body_parameters = {}
        body_parameters["otpauthenticator"] = otpauthenticator

        resource_path = "identity/v2/auth/account/2fa/authenticator"
        return self._lr_object.execute("DELETE", resource_path, query_parameters, body_parameters)

    def mfa_backup_code_by_access_token(self, access_token):
        """This API is used to get a set of backup codes via access token to allow the user login on a site that has Multi-factor Authentication enabled in the event that the user does not have a secondary factor available. We generate 10 codes, each code can only be consumed once. If any user attempts to go over the number of invalid login attempts configured in the Dashboard then the account gets blocked automatically
        
        Args:
            access_token: Uniquely generated identifier key by LoginRadius that is activated after successful authentication.
		
        Returns:
            Response containing Definition of Complete Backup Code data
        5.13
        """

        if(self._lr_object.is_null_or_whitespace(access_token)):
            raise Exception(self._lr_object.get_validation_message("access_token"))

        query_parameters = {}
        query_parameters["access_token"] = access_token
        query_parameters["apiKey"] = self._lr_object.get_api_key()

        resource_path = "identity/v2/auth/account/2fa/backupcode"
        return self._lr_object.execute("GET", resource_path, query_parameters, {})

    def mfa_reset_backup_code_by_access_token(self, access_token):
        """API is used to reset the backup codes on a given account via the access token. This API call will generate 10 new codes, each code can only be consumed once
        
        Args:
            access_token: Uniquely generated identifier key by LoginRadius that is activated after successful authentication.
		
        Returns:
            Response containing Definition of Complete Backup Code data
        5.14
        """

        if(self._lr_object.is_null_or_whitespace(access_token)):
            raise Exception(self._lr_object.get_validation_message("access_token"))

        query_parameters = {}
        query_parameters["access_token"] = access_token
        query_parameters["apiKey"] = self._lr_object.get_api_key()

        resource_path = "identity/v2/auth/account/2fa/backupcode/reset"
        return self._lr_object.execute("GET", resource_path, query_parameters, {})

    def mfa_email_otp_by_access_token(self, access_token, email_id, email_template2_f_a=None):
        """This API is created to send the OTP to the email if email OTP authenticator is enabled in app's MFA configuration.
        
        Args:
            access_token: access_token
            email_id: EmailId
            email_template2_f_a: EmailTemplate2FA
		
        Returns:
            Response containing Definition of Complete Validation data
        5.17
        """

        if(self._lr_object.is_null_or_whitespace(access_token)):
            raise Exception(self._lr_object.get_validation_message("access_token"))

        if(self._lr_object.is_null_or_whitespace(email_id)):
            raise Exception(self._lr_object.get_validation_message("email_id"))

        query_parameters = {}
        query_parameters["access_token"] = access_token
        query_parameters["apiKey"] = self._lr_object.get_api_key()
        query_parameters["emailId"] = email_id
        if(not self._lr_object.is_null_or_whitespace(email_template2_f_a)):
            query_parameters["emailTemplate2FA"] = email_template2_f_a

        resource_path = "identity/v2/auth/account/2fa/otp/email"
        return self._lr_object.execute("GET", resource_path, query_parameters, {})

    def mfa_validate_email_otp_by_access_token(self, access_token, multi_factor_auth_model_by_email_otp_with_lockout):
        """This API is used to set up MFA Email OTP authenticator on profile after login.
        
        Args:
            access_token: access_token
            multi_factor_auth_model_by_email_otp_with_lockout: payload
		
        Returns:
            Response containing Definition for Complete profile data
        5.18
        """

        if(self._lr_object.is_null_or_whitespace(access_token)):
            raise Exception(self._lr_object.get_validation_message("access_token"))
        if(multi_factor_auth_model_by_email_otp_with_lockout is None):
            raise Exception(self._lr_object.get_validation_message("multi_factor_auth_model_by_email_otp_with_lockout"))

        query_parameters = {}
        query_parameters["access_token"] = access_token
        query_parameters["apiKey"] = self._lr_object.get_api_key()

        resource_path = "identity/v2/auth/account/2fa/verification/otp/email"
        return self._lr_object.execute("PUT", resource_path, query_parameters, multi_factor_auth_model_by_email_otp_with_lockout)

    def mfa_reset_email_otp_authenticator_by_access_token(self, access_token):
        """This API is used to reset the Email OTP Authenticator settings for an MFA-enabled user
        
        Args:
            access_token: access_token
		
        Returns:
            Response containing Definition of Delete Request
        5.19
        """

        if(self._lr_object.is_null_or_whitespace(access_token)):
            raise Exception(self._lr_object.get_validation_message("access_token"))

        query_parameters = {}
        query_parameters["access_token"] = access_token
        query_parameters["apiKey"] = self._lr_object.get_api_key()

        resource_path = "identity/v2/auth/account/2fa/authenticator/otp/email"
        return self._lr_object.execute("DELETE", resource_path, query_parameters, {})

    def mfa_security_question_answer_by_access_token(self, access_token, security_question_answer_model_by_access_token):
        """This API is used to set up MFA Security Question authenticator on profile after login.
        
        Args:
            access_token: access_token
            security_question_answer_model_by_access_token: payload
		
        Returns:
            Response containing Definition of Complete Validation data
        5.20
        """

        if(self._lr_object.is_null_or_whitespace(access_token)):
            raise Exception(self._lr_object.get_validation_message("access_token"))
        if(security_question_answer_model_by_access_token is None):
            raise Exception(self._lr_object.get_validation_message("security_question_answer_model_by_access_token"))

        query_parameters = {}
        query_parameters["access_token"] = access_token
        query_parameters["apiKey"] = self._lr_object.get_api_key()

        resource_path = "identity/v2/auth/account/2fa/securityquestionanswer"
        return self._lr_object.execute("PUT", resource_path, query_parameters, security_question_answer_model_by_access_token)

    def mfa_reset_security_question_authenticator_by_access_token(self, access_token):
        """This API is used to Reset MFA Security Question Authenticator By Access Token
        
        Args:
            access_token: access_token
		
        Returns:
            Response containing Definition of Delete Request
        5.21
        """

        if(self._lr_object.is_null_or_whitespace(access_token)):
            raise Exception(self._lr_object.get_validation_message("access_token"))

        query_parameters = {}
        query_parameters["access_token"] = access_token
        query_parameters["apiKey"] = self._lr_object.get_api_key()

        resource_path = "identity/v2/auth/account/2fa/authenticator/securityquestionanswer"
        return self._lr_object.execute("DELETE", resource_path, query_parameters, {})

    def mfa_login_by_email(self, email, password, email_template=None, fields='',
        login_url=None, sms_template=None,sms_template2_f_a=None, 
        verification_url=None,email_template2_f_a=None):
        """This API can be used to login by emailid on a Multi-factor authentication enabled LoginRadius site.
        
        Args:
            email: user's email
            password: Password for the email
            email_template: Email template name
            fields: The fields parameter filters the API response so that the response only includes a specific set of fields
            login_url: Url where the user is logging from
            sms_template: SMS Template name
            sms_template2_f_a: SMS Template Name
            verification_url: Email verification url
            email_template2_f_a: 2FA Email Template name

		
        Returns:
            Complete user UserProfile data
        9.8.1
        """

        if(self._lr_object.is_null_or_whitespace(email)):
            raise Exception(self._lr_object.get_validation_message("email"))

        if(self._lr_object.is_null_or_whitespace(password)):
            raise Exception(self._lr_object.get_validation_message("password"))

        query_parameters = {}
        query_parameters["apiKey"] = self._lr_object.get_api_key()
        if(not self._lr_object.is_null_or_whitespace(email_template)):
            query_parameters["emailTemplate"] = email_template
        if(not self._lr_object.is_null_or_whitespace(fields)):
            query_parameters["fields"] = fields
        if(not self._lr_object.is_null_or_whitespace(login_url)):
            query_parameters["loginUrl"] = login_url
        if(not self._lr_object.is_null_or_whitespace(sms_template)):
            query_parameters["smsTemplate"] = sms_template
        if(not self._lr_object.is_null_or_whitespace(sms_template2_f_a)):
            query_parameters["smsTemplate2FA"] = sms_template2_f_a
        if(not self._lr_object.is_null_or_whitespace(verification_url)):
            query_parameters["verificationUrl"] = verification_url
        if(not self._lr_object.is_null_or_whitespace(email_template2_f_a)):
            query_parameters["emailTemplate2FA"] = email_template2_f_a

        body_parameters = {}
        body_parameters["email"] = email
        body_parameters["password"] = password

        resource_path = "identity/v2/auth/login/2fa"
        return self._lr_object.execute("POST", resource_path, query_parameters, body_parameters)

    def mfa_login_by_user_name(self, password, username, email_template=None, fields='',
        login_url=None, sms_template=None,sms_template2_f_a=None,
        verification_url=None, email_template2_f_a=None):
        """This API can be used to login by username on a Multi-factor authentication enabled LoginRadius site.
        
        Args:
            password: Password for the email
            username: Username of the user
            email_template: Email template name
            fields: The fields parameter filters the API response so that the response only includes a specific set of fields
            login_url: Url where the user is logging from
            sms_template: SMS Template name
            sms_template2_f_a: SMS Template Name
            verification_url: Email verification url
            email_template2_f_a: 2FA Email Template name

		
        Returns:
            Complete user UserProfile data
        9.8.2
        """

        if(self._lr_object.is_null_or_whitespace(password)):
            raise Exception(self._lr_object.get_validation_message("password"))

        if(self._lr_object.is_null_or_whitespace(username)):
            raise Exception(self._lr_object.get_validation_message("username"))

        query_parameters = {}
        query_parameters["apiKey"] = self._lr_object.get_api_key()
        if(not self._lr_object.is_null_or_whitespace(email_template)):
            query_parameters["emailTemplate"] = email_template
        if(not self._lr_object.is_null_or_whitespace(fields)):
            query_parameters["fields"] = fields
        if(not self._lr_object.is_null_or_whitespace(login_url)):
            query_parameters["loginUrl"] = login_url
        if(not self._lr_object.is_null_or_whitespace(sms_template)):
            query_parameters["smsTemplate"] = sms_template
        if(not self._lr_object.is_null_or_whitespace(sms_template2_f_a)):
            query_parameters["smsTemplate2FA"] = sms_template2_f_a
        if(not self._lr_object.is_null_or_whitespace(verification_url)):
            query_parameters["verificationUrl"] = verification_url
        if(not self._lr_object.is_null_or_whitespace(email_template2_f_a)):
            query_parameters["emailTemplate2FA"] = email_template2_f_a

        body_parameters = {}
        body_parameters["password"] = password
        body_parameters["username"] = username

        resource_path = "identity/v2/auth/login/2fa"
        return self._lr_object.execute("POST", resource_path, query_parameters, body_parameters)

    def mfa_login_by_phone(self, password, phone, email_template=None,
        fields='', login_url=None, sms_template=None,sms_template2_f_a=None,
        verification_url=None, email_template2_f_a=None,):
        """This API can be used to login by Phone on a Multi-factor authentication enabled LoginRadius site.
        
        Args:
            password: Password for the email
            phone: New Phone Number
            email_template: Email template name
            fields: The fields parameter filters the API response so that the response only includes a specific set of fields
            login_url: Url where the user is logging from
            sms_template: SMS Template name
            sms_template2_f_a: SMS Template Name
            verification_url: Email verification url
            email_template2_f_a: 2FA Email Template name

		
        Returns:
            Complete user UserProfile data
        9.8.3
        """

        if(self._lr_object.is_null_or_whitespace(password)):
            raise Exception(self._lr_object.get_validation_message("password"))

        if(self._lr_object.is_null_or_whitespace(phone)):
            raise Exception(self._lr_object.get_validation_message("phone"))

        query_parameters = {}
        query_parameters["apiKey"] = self._lr_object.get_api_key()
        if(not self._lr_object.is_null_or_whitespace(email_template)):
            query_parameters["emailTemplate"] = email_template
        if(not self._lr_object.is_null_or_whitespace(fields)):
            query_parameters["fields"] = fields
        if(not self._lr_object.is_null_or_whitespace(login_url)):
            query_parameters["loginUrl"] = login_url
        if(not self._lr_object.is_null_or_whitespace(sms_template)):
            query_parameters["smsTemplate"] = sms_template
        if(not self._lr_object.is_null_or_whitespace(sms_template2_f_a)):
            query_parameters["smsTemplate2FA"] = sms_template2_f_a
        if(not self._lr_object.is_null_or_whitespace(verification_url)):
            query_parameters["verificationUrl"] = verification_url
        if(not self._lr_object.is_null_or_whitespace(email_template2_f_a)):
            query_parameters["emailTemplate2FA"] = email_template2_f_a

        body_parameters = {}
        body_parameters["password"] = password
        body_parameters["phone"] = phone

        resource_path = "identity/v2/auth/login/2fa"
        return self._lr_object.execute("POST", resource_path, query_parameters, body_parameters)

    def mfa_validate_otp_by_phone(self, multi_factor_auth_model_with_lockout, second_factor_authentication_token, fields='', 
        sms_template2_f_a=None, rba_browser_email_template=None, rba_city_email_template=None, rba_country_email_template=None, 
        rba_ip_email_template=None):
        """This API is used to login via Multi-factor authentication by passing the One Time Password received via SMS
        
        Args:
            multi_factor_auth_model_with_lockout: Model Class containing Definition of payload for MultiFactorAuthModel With Lockout API
            second_factor_authentication_token: A Uniquely generated MFA identifier token after successful authentication
            fields: The fields parameter filters the API response so that the response only includes a specific set of fields
            sms_template2_f_a: SMS Template Name
            rba_browser_email_template: 
            rba_city_email_template: 
            rba_country_email_template: 
            rba_ip_email_template: 
		
        Returns:
            Complete user UserProfile data
        9.12
        """
        if(multi_factor_auth_model_with_lockout is None):
            raise Exception(self._lr_object.get_validation_message("multi_factor_auth_model_with_lockout"))

        if(self._lr_object.is_null_or_whitespace(second_factor_authentication_token)):
            raise Exception(self._lr_object.get_validation_message("second_factor_authentication_token"))

        query_parameters = {}
        query_parameters["apiKey"] = self._lr_object.get_api_key()
        query_parameters["secondFactorAuthenticationToken"] = second_factor_authentication_token
        if(not self._lr_object.is_null_or_whitespace(fields)):
            query_parameters["fields"] = fields
        if(not self._lr_object.is_null_or_whitespace(rba_browser_email_template)):
            query_parameters["rbaBrowserEmailTemplate"] = rba_browser_email_template
        if(not self._lr_object.is_null_or_whitespace(rba_city_email_template)):
            query_parameters["rbaCityEmailTemplate"] = rba_city_email_template
        if(not self._lr_object.is_null_or_whitespace(rba_country_email_template)):
            query_parameters["rbaCountryEmailTemplate"] = rba_country_email_template
        if(not self._lr_object.is_null_or_whitespace(rba_ip_email_template)):
            query_parameters["rbaIpEmailTemplate"] = rba_ip_email_template
        if(not self._lr_object.is_null_or_whitespace(sms_template2_f_a)):
            query_parameters["smsTemplate2FA"] = sms_template2_f_a

        resource_path = "identity/v2/auth/login/2fa/verification/otp"
        return self._lr_object.execute("PUT", resource_path, query_parameters, multi_factor_auth_model_with_lockout)

    def mfa_validate_google_auth_code(self, google_authenticator_code, second_factor_authentication_token, fields='',
        rba_browser_email_template=None, rba_city_email_template=None, rba_country_email_template=None, rba_ip_email_template=None):
        """This API is used to login via Multi-factor-authentication by passing the google authenticator code.
        
        Args:
            google_authenticator_code: The code generated by google authenticator app after scanning QR code
            second_factor_authentication_token: SecondFactorAuthenticationToken
            fields: The fields parameter filters the API response so that the response only includes a specific set of fields
            rba_browser_email_template: RbaBrowserEmailTemplate
            rba_city_email_template: RbaCityEmailTemplate
            rba_country_email_template: RbaCountryEmailTemplate
            rba_ip_email_template: RbaIpEmailTemplate
		
        Returns:
            Complete user UserProfile data
        9.13
        """

        if(self._lr_object.is_null_or_whitespace(google_authenticator_code)):
            raise Exception(self._lr_object.get_validation_message("google_authenticator_code"))

        if(self._lr_object.is_null_or_whitespace(second_factor_authentication_token)):
            raise Exception(self._lr_object.get_validation_message("second_factor_authentication_token"))

        query_parameters = {}
        query_parameters["apiKey"] = self._lr_object.get_api_key()
        query_parameters["secondFactorAuthenticationToken"] = second_factor_authentication_token
        if(not self._lr_object.is_null_or_whitespace(fields)):
            query_parameters["fields"] = fields
        if(not self._lr_object.is_null_or_whitespace(rba_browser_email_template)):
            query_parameters["rbaBrowserEmailTemplate"] = rba_browser_email_template
        if(not self._lr_object.is_null_or_whitespace(rba_city_email_template)):
            query_parameters["rbaCityEmailTemplate"] = rba_city_email_template
        if(not self._lr_object.is_null_or_whitespace(rba_country_email_template)):
            query_parameters["rbaCountryEmailTemplate"] = rba_country_email_template
        if(not self._lr_object.is_null_or_whitespace(rba_ip_email_template)):
            query_parameters["rbaIpEmailTemplate"] = rba_ip_email_template

        body_parameters = {}
        body_parameters["googleAuthenticatorCode"] = google_authenticator_code

        resource_path = "identity/v2/auth/login/2fa/verification/googleauthenticatorcode"
        return self._lr_object.execute("PUT", resource_path, query_parameters, body_parameters)

    def mfa_validate_backup_code(self, multi_factor_auth_model_by_backup_code, second_factor_authentication_token, fields='',
        rba_browser_email_template=None, rba_city_email_template=None, rba_country_email_template=None, rba_ip_email_template=None):
        """This API is used to validate the backup code provided by the user and if valid, we return an access token allowing the user to login incases where Multi-factor authentication (MFA) is enabled and the secondary factor is unavailable. When a user initially downloads the Backup codes, We generate 10 codes, each code can only be consumed once. if any user attempts to go over the number of invalid login attempts configured in the Dashboard then the account gets blocked automatically
        
        Args:
            multi_factor_auth_model_by_backup_code: Model Class containing Definition of payload for MultiFactorAuth By BackupCode API
            second_factor_authentication_token: A Uniquely generated MFA identifier token after successful authentication
            fields: The fields parameter filters the API response so that the response only includes a specific set of fields
            rba_browser_email_template: 
            rba_city_email_template: 
            rba_country_email_template: 
            rba_ip_email_template: 
		
        Returns:
            Complete user UserProfile data
        9.14
        """
        if(multi_factor_auth_model_by_backup_code is None):
            raise Exception(self._lr_object.get_validation_message("multi_factor_auth_model_by_backup_code"))

        if(self._lr_object.is_null_or_whitespace(second_factor_authentication_token)):
            raise Exception(self._lr_object.get_validation_message("second_factor_authentication_token"))

        query_parameters = {}
        query_parameters["apiKey"] = self._lr_object.get_api_key()
        query_parameters["secondFactorAuthenticationToken"] = second_factor_authentication_token
        if(not self._lr_object.is_null_or_whitespace(fields)):
            query_parameters["fields"] = fields
        if(not self._lr_object.is_null_or_whitespace(rba_browser_email_template)):
            query_parameters["rbaBrowserEmailTemplate"] = rba_browser_email_template
        if(not self._lr_object.is_null_or_whitespace(rba_city_email_template)):
            query_parameters["rbaCityEmailTemplate"] = rba_city_email_template
        if(not self._lr_object.is_null_or_whitespace(rba_country_email_template)):
            query_parameters["rbaCountryEmailTemplate"] = rba_country_email_template
        if(not self._lr_object.is_null_or_whitespace(rba_ip_email_template)):
            query_parameters["rbaIpEmailTemplate"] = rba_ip_email_template

        resource_path = "identity/v2/auth/login/2fa/verification/backupcode"
        return self._lr_object.execute("PUT", resource_path, query_parameters, multi_factor_auth_model_by_backup_code)

    def mfa_update_phone_number(self, phone_no2_f_a, second_factor_authentication_token, sms_template2_f_a=None):
        """This API is used to update (if configured) the phone number used for Multi-factor authentication by sending the verification OTP to the provided phone number
        
        Args:
            phone_no2_f_a: Phone Number For 2FA
            second_factor_authentication_token: A Uniquely generated MFA identifier token after successful authentication
            sms_template2_f_a: SMS Template Name
		
        Returns:
            Response containing Definition for Complete SMS data
        9.16
        """

        if(self._lr_object.is_null_or_whitespace(phone_no2_f_a)):
            raise Exception(self._lr_object.get_validation_message("phone_no2_f_a"))

        if(self._lr_object.is_null_or_whitespace(second_factor_authentication_token)):
            raise Exception(self._lr_object.get_validation_message("second_factor_authentication_token"))

        query_parameters = {}
        query_parameters["apiKey"] = self._lr_object.get_api_key()
        query_parameters["secondFactorAuthenticationToken"] = second_factor_authentication_token
        if(not self._lr_object.is_null_or_whitespace(sms_template2_f_a)):
            query_parameters["smsTemplate2FA"] = sms_template2_f_a

        body_parameters = {}
        body_parameters["phoneNo2FA"] = phone_no2_f_a

        resource_path = "identity/v2/auth/login/2fa"
        return self._lr_object.execute("PUT", resource_path, query_parameters, body_parameters)

    def mfa_resend_otp(self, second_factor_authentication_token, sms_template2_f_a=None):
        """This API is used to resending the verification OTP to the provided phone number
        
        Args:
            second_factor_authentication_token: A Uniquely generated MFA identifier token after successful authentication
            sms_template2_f_a: SMS Template Name
		
        Returns:
            Response containing Definition for Complete SMS data
        9.17
        """

        if(self._lr_object.is_null_or_whitespace(second_factor_authentication_token)):
            raise Exception(self._lr_object.get_validation_message("second_factor_authentication_token"))

        query_parameters = {}
        query_parameters["apiKey"] = self._lr_object.get_api_key()
        query_parameters["secondFactorAuthenticationToken"] = second_factor_authentication_token
        if(not self._lr_object.is_null_or_whitespace(sms_template2_f_a)):
            query_parameters["smsTemplate2FA"] = sms_template2_f_a

        resource_path = "identity/v2/auth/login/2fa/resend"
        return self._lr_object.execute("GET", resource_path, query_parameters, {})

    def mfa_email_otp(self, email_id_model, second_factor_authentication_token, email_template2_f_a=None):
        """An API designed to send the MFA Email OTP to the email.
        
        Args:
            email_id_model: payload
            second_factor_authentication_token: SecondFactorAuthenticationToken
            email_template2_f_a: EmailTemplate2FA
		
        Returns:
            Response containing Definition of Complete Validation data
        9.18
        """
        if(email_id_model is None):
            raise Exception(self._lr_object.get_validation_message("email_id_model"))

        if(self._lr_object.is_null_or_whitespace(second_factor_authentication_token)):
            raise Exception(self._lr_object.get_validation_message("second_factor_authentication_token"))

        query_parameters = {}
        query_parameters["apiKey"] = self._lr_object.get_api_key()
        query_parameters["secondFactorAuthenticationToken"] = second_factor_authentication_token
        if(not self._lr_object.is_null_or_whitespace(email_template2_f_a)):
            query_parameters["emailTemplate2FA"] = email_template2_f_a

        resource_path = "identity/v2/auth/login/2fa/otp/email"
        return self._lr_object.execute("POST", resource_path, query_parameters, email_id_model)

    def mfa_validate_email_otp(self, multi_factor_auth_model_by_email_otp, second_factor_authentication_token, rba_browser_email_template=None,
        rba_city_email_template=None, rba_country_email_template=None, rba_ip_email_template=None):
        """This API is used to Verify MFA Email OTP by MFA Token
        
        Args:
            multi_factor_auth_model_by_email_otp: payload
            second_factor_authentication_token: SecondFactorAuthenticationToken
            rba_browser_email_template: RbaBrowserEmailTemplate
            rba_city_email_template: RbaCityEmailTemplate
            rba_country_email_template: RbaCountryEmailTemplate
            rba_ip_email_template: RbaIpEmailTemplate
		
        Returns:
            Response Containing Access Token and Complete Profile Data
        9.25
        """
        if(multi_factor_auth_model_by_email_otp is None):
            raise Exception(self._lr_object.get_validation_message("multi_factor_auth_model_by_email_otp"))

        if(self._lr_object.is_null_or_whitespace(second_factor_authentication_token)):
            raise Exception(self._lr_object.get_validation_message("second_factor_authentication_token"))

        query_parameters = {}
        query_parameters["apiKey"] = self._lr_object.get_api_key()
        query_parameters["secondFactorAuthenticationToken"] = second_factor_authentication_token
        if(not self._lr_object.is_null_or_whitespace(rba_browser_email_template)):
            query_parameters["rbaBrowserEmailTemplate"] = rba_browser_email_template
        if(not self._lr_object.is_null_or_whitespace(rba_city_email_template)):
            query_parameters["rbaCityEmailTemplate"] = rba_city_email_template
        if(not self._lr_object.is_null_or_whitespace(rba_country_email_template)):
            query_parameters["rbaCountryEmailTemplate"] = rba_country_email_template
        if(not self._lr_object.is_null_or_whitespace(rba_ip_email_template)):
            query_parameters["rbaIpEmailTemplate"] = rba_ip_email_template

        resource_path = "identity/v2/auth/login/2fa/verification/otp/email"
        return self._lr_object.execute("PUT", resource_path, query_parameters, multi_factor_auth_model_by_email_otp)

    def mfa_security_question_answer(self, security_question_answer_update_model, second_factor_authentication_token):
        """This API is used to set the security questions on the profile with the MFA token when MFA flow is required.
        
        Args:
            security_question_answer_update_model: payload
            second_factor_authentication_token: SecondFactorAuthenticationToken
		
        Returns:
            Response Containing Access Token and Complete Profile Data
        9.26
        """
        if(security_question_answer_update_model is None):
            raise Exception(self._lr_object.get_validation_message("security_question_answer_update_model"))

        if(self._lr_object.is_null_or_whitespace(second_factor_authentication_token)):
            raise Exception(self._lr_object.get_validation_message("second_factor_authentication_token"))

        query_parameters = {}
        query_parameters["apiKey"] = self._lr_object.get_api_key()
        query_parameters["secondFactorAuthenticationToken"] = second_factor_authentication_token

        resource_path = "identity/v2/auth/login/2fa/securityquestionanswer"
        return self._lr_object.execute("PUT", resource_path, query_parameters, security_question_answer_update_model)

    def mfa_security_question_answer_verification(self, security_question_answer_update_model, second_factor_authentication_token, rba_browser_email_template=None,
        rba_city_email_template=None, rba_country_email_template=None, rba_ip_email_template=None):
        """This API is used to resending the verification OTP to the provided phone number
        
        Args:
            security_question_answer_update_model: payload
            second_factor_authentication_token: SecondFactorAuthenticationToken
            rba_browser_email_template: RbaBrowserEmailTemplate
            rba_city_email_template: RbaCityEmailTemplate
            rba_country_email_template: RbaCountryEmailTemplate
            rba_ip_email_template: RbaIpEmailTemplate
		
        Returns:
            Response Containing Access Token and Complete Profile Data
        9.27
        """
        if(security_question_answer_update_model is None):
            raise Exception(self._lr_object.get_validation_message("security_question_answer_update_model"))

        if(self._lr_object.is_null_or_whitespace(second_factor_authentication_token)):
            raise Exception(self._lr_object.get_validation_message("second_factor_authentication_token"))

        query_parameters = {}
        query_parameters["apiKey"] = self._lr_object.get_api_key()
        query_parameters["secondFactorAuthenticationToken"] = second_factor_authentication_token
        if(not self._lr_object.is_null_or_whitespace(rba_browser_email_template)):
            query_parameters["rbaBrowserEmailTemplate"] = rba_browser_email_template
        if(not self._lr_object.is_null_or_whitespace(rba_city_email_template)):
            query_parameters["rbaCityEmailTemplate"] = rba_city_email_template
        if(not self._lr_object.is_null_or_whitespace(rba_country_email_template)):
            query_parameters["rbaCountryEmailTemplate"] = rba_country_email_template
        if(not self._lr_object.is_null_or_whitespace(rba_ip_email_template)):
            query_parameters["rbaIpEmailTemplate"] = rba_ip_email_template

        resource_path = "identity/v2/auth/login/2fa/verification/securityquestionanswer"
        return self._lr_object.execute("POST", resource_path, query_parameters, security_question_answer_update_model)

    def mfa_reset_sms_authenticator_by_uid(self, otpauthenticator, uid):
        """This API resets the SMS Authenticator configurations on a given account via the UID.
        
        Args:
            otpauthenticator: Pass 'otpauthenticator' to remove SMS Authenticator
            uid: UID, the unified identifier for each user account
		
        Returns:
            Response containing Definition of Delete Request
        18.21.1
        """

        if(self._lr_object.is_null_or_whitespace(uid)):
            raise Exception(self._lr_object.get_validation_message("uid"))

        query_parameters = {}
        query_parameters["apiKey"] = self._lr_object.get_api_key()
        query_parameters["apiSecret"] = self._lr_object.get_api_secret()
        query_parameters["uid"] = uid

        body_parameters = {}
        body_parameters["otpauthenticator"] = otpauthenticator

        resource_path = "identity/v2/manage/account/2fa/authenticator"
        return self._lr_object.execute("DELETE", resource_path, query_parameters, body_parameters)

    def mfa_reset_google_authenticator_by_uid(self, googleauthenticator, uid):
        """This API resets the Google Authenticator configurations on a given account via the UID.
        
        Args:
            googleauthenticator: boolean type value,Enable google Authenticator Code.
            uid: UID, the unified identifier for each user account
		
        Returns:
            Response containing Definition of Delete Request
        18.21.2
        """

        if(self._lr_object.is_null_or_whitespace(uid)):
            raise Exception(self._lr_object.get_validation_message("uid"))

        query_parameters = {}
        query_parameters["apiKey"] = self._lr_object.get_api_key()
        query_parameters["apiSecret"] = self._lr_object.get_api_secret()
        query_parameters["uid"] = uid

        body_parameters = {}
        body_parameters["googleauthenticator"] = googleauthenticator

        resource_path = "identity/v2/manage/account/2fa/authenticator"
        return self._lr_object.execute("DELETE", resource_path, query_parameters, body_parameters)

    def mfa_backup_code_by_uid(self, uid):
        """This API is used to reset the backup codes on a given account via the UID. This API call will generate 10 new codes, each code can only be consumed once.
        
        Args:
            uid: UID, the unified identifier for each user account
		
        Returns:
            Response containing Definition of Complete Backup Code data
        18.25
        """

        if(self._lr_object.is_null_or_whitespace(uid)):
            raise Exception(self._lr_object.get_validation_message("uid"))

        query_parameters = {}
        query_parameters["apiKey"] = self._lr_object.get_api_key()
        query_parameters["apiSecret"] = self._lr_object.get_api_secret()
        query_parameters["uid"] = uid

        resource_path = "identity/v2/manage/account/2fa/backupcode"
        return self._lr_object.execute("GET", resource_path, query_parameters, {})

    def mfa_reset_backup_code_by_uid(self, uid):
        """This API is used to reset the backup codes on a given account via the UID. This API call will generate 10 new codes, each code can only be consumed once.
        
        Args:
            uid: UID, the unified identifier for each user account
		
        Returns:
            Response containing Definition of Complete Backup Code data
        18.26
        """

        if(self._lr_object.is_null_or_whitespace(uid)):
            raise Exception(self._lr_object.get_validation_message("uid"))

        query_parameters = {}
        query_parameters["apiKey"] = self._lr_object.get_api_key()
        query_parameters["apiSecret"] = self._lr_object.get_api_secret()
        query_parameters["uid"] = uid

        resource_path = "identity/v2/manage/account/2fa/backupcode/reset"
        return self._lr_object.execute("GET", resource_path, query_parameters, {})

    def mfa_reset_email_otp_authenticator_by_uid(self, uid):
        """This API is used to reset the Email OTP Authenticator settings for an MFA-enabled user.
        
        Args:
            uid: UID, the unified identifier for each user account
		
        Returns:
            Response containing Definition of Delete Request
        18.42
        """

        if(self._lr_object.is_null_or_whitespace(uid)):
            raise Exception(self._lr_object.get_validation_message("uid"))

        query_parameters = {}
        query_parameters["apiKey"] = self._lr_object.get_api_key()
        query_parameters["apiSecret"] = self._lr_object.get_api_secret()
        query_parameters["uid"] = uid

        resource_path = "identity/v2/manage/account/2fa/authenticator/otp/email"
        return self._lr_object.execute("DELETE", resource_path, query_parameters, {})

    def mfa_reset_security_question_authenticator_by_uid(self, uid):
        """This API is used to reset the Security Question Authenticator settings for an MFA-enabled user.
        
        Args:
            uid: UID, the unified identifier for each user account
		
        Returns:
            Response containing Definition of Delete Request
        18.43
        """

        if(self._lr_object.is_null_or_whitespace(uid)):
            raise Exception(self._lr_object.get_validation_message("uid"))

        query_parameters = {}
        query_parameters["apiKey"] = self._lr_object.get_api_key()
        query_parameters["apiSecret"] = self._lr_object.get_api_secret()
        query_parameters["uid"] = uid

        resource_path = "identity/v2/manage/account/2fa/authenticator/securityquestionanswer"
        return self._lr_object.execute("DELETE", resource_path, query_parameters, {})



# -- coding: utf-8 --
# Created by LoginRadius Development Team
# Copyright 2019 LoginRadius Inc. All rights reserved.
#


class AccountApi:

    def __init__(self, lr_object):
        """
        :param lr_object: this is the reference to the parent LoginRadius object.
        """
        self._lr_object = lr_object

    def get_privacy_policy_history_by_uid(self, uid):
        """This API is used to retrieve all of the accepted Policies by the user, associated with their UID.
        
        Args:
            uid: UID, the unified identifier for each user account
		
        Returns:
            Complete Policy History data
        15.1.1
        """

        if(self._lr_object.is_null_or_whitespace(uid)):
            raise Exception(self._lr_object.get_validation_message("uid"))

        query_parameters = {}
        query_parameters["apiKey"] = self._lr_object.get_api_key()
        query_parameters["apiSecret"] = self._lr_object.get_api_secret()

        resource_path = "identity/v2/manage/account/" + uid + "/privacypolicy/history"
        return self._lr_object.execute("GET", resource_path, query_parameters, {})

    def create_account(self, account_create_model, fields=''):
        """This API is used to create an account in Cloud Storage. This API bypass the normal email verification process and manually creates the user. <br><br>In order to use this API, you need to format a JSON request body with all of the mandatory fields
        
        Args:
            account_create_model: Model Class containing Definition of payload for Account Create API
            fields: The fields parameter filters the API response so that the response only includes a specific set of fields
		
        Returns:
            Response containing Definition for Complete profile data
        18.1
        """
        if(account_create_model is None):
            raise Exception(self._lr_object.get_validation_message("account_create_model"))

        query_parameters = {}
        query_parameters["apiKey"] = self._lr_object.get_api_key()
        query_parameters["apiSecret"] = self._lr_object.get_api_secret()
        if(not self._lr_object.is_null_or_whitespace(fields)):
            query_parameters["fields"] = fields

        resource_path = "identity/v2/manage/account"
        return self._lr_object.execute("POST", resource_path, query_parameters, account_create_model)

    def get_account_profile_by_email(self, email, fields=''):
        """This API is used to retrieve all of the profile data, associated with the specified account by email in Cloud Storage.
        
        Args:
            email: Email of the user
            fields: The fields parameter filters the API response so that the response only includes a specific set of fields
		
        Returns:
            Response containing Definition for Complete profile data
        18.2
        """

        if(self._lr_object.is_null_or_whitespace(email)):
            raise Exception(self._lr_object.get_validation_message("email"))

        query_parameters = {}
        query_parameters["apiKey"] = self._lr_object.get_api_key()
        query_parameters["apiSecret"] = self._lr_object.get_api_secret()
        query_parameters["email"] = email
        if(not self._lr_object.is_null_or_whitespace(fields)):
            query_parameters["fields"] = fields

        resource_path = "identity/v2/manage/account"
        return self._lr_object.execute("GET", resource_path, query_parameters, {})

    def get_account_profile_by_user_name(self, user_name, fields=''):
        """This API is used to retrieve all of the profile data associated with the specified account by user name in Cloud Storage.
        
        Args:
            user_name: UserName of the user
            fields: The fields parameter filters the API response so that the response only includes a specific set of fields
		
        Returns:
            Response containing Definition for Complete profile data
        18.3
        """

        if(self._lr_object.is_null_or_whitespace(user_name)):
            raise Exception(self._lr_object.get_validation_message("user_name"))

        query_parameters = {}
        query_parameters["apiKey"] = self._lr_object.get_api_key()
        query_parameters["apiSecret"] = self._lr_object.get_api_secret()
        query_parameters["userName"] = user_name
        if(not self._lr_object.is_null_or_whitespace(fields)):
            query_parameters["fields"] = fields

        resource_path = "identity/v2/manage/account"
        return self._lr_object.execute("GET", resource_path, query_parameters, {})

    def get_account_profile_by_phone(self, phone, fields=''):
        """This API is used to retrieve all of the profile data, associated with the account by phone number in Cloud Storage.
        
        Args:
            phone: The Registered Phone Number
            fields: The fields parameter filters the API response so that the response only includes a specific set of fields
		
        Returns:
            Response containing Definition for Complete profile data
        18.4
        """

        if(self._lr_object.is_null_or_whitespace(phone)):
            raise Exception(self._lr_object.get_validation_message("phone"))

        query_parameters = {}
        query_parameters["apiKey"] = self._lr_object.get_api_key()
        query_parameters["apiSecret"] = self._lr_object.get_api_secret()
        query_parameters["phone"] = phone
        if(not self._lr_object.is_null_or_whitespace(fields)):
            query_parameters["fields"] = fields

        resource_path = "identity/v2/manage/account"
        return self._lr_object.execute("GET", resource_path, query_parameters, {})

    def get_account_profile_by_uid(self, uid, fields=''):
        """This API is used to retrieve all of the profile data, associated with the account by uid in Cloud Storage.
        
        Args:
            uid: UID, the unified identifier for each user account
            fields: The fields parameter filters the API response so that the response only includes a specific set of fields
		
        Returns:
            Response containing Definition for Complete profile data
        18.5
        """

        if(self._lr_object.is_null_or_whitespace(uid)):
            raise Exception(self._lr_object.get_validation_message("uid"))

        query_parameters = {}
        query_parameters["apiKey"] = self._lr_object.get_api_key()
        query_parameters["apiSecret"] = self._lr_object.get_api_secret()
        if(not self._lr_object.is_null_or_whitespace(fields)):
            query_parameters["fields"] = fields

        resource_path = "identity/v2/manage/account/" + uid
        return self._lr_object.execute("GET", resource_path, query_parameters, {})

    def update_account_by_uid(self, account_user_profile_update_model, uid, fields='',
        null_support=None):
        """This API is used to update the information of existing accounts in your Cloud Storage. See our Advanced API Usage section <a href='https://www.loginradius.com/docs/api/v2/customer-identity-api/advanced-api-usage/'>Here</a> for more capabilities.
        
        Args:
            account_user_profile_update_model: Model Class containing Definition of payload for Account Update API
            uid: UID, the unified identifier for each user account
            fields: The fields parameter filters the API response so that the response only includes a specific set of fields
            null_support: Boolean, pass true if you wish to update any user profile field with a NULL value, You can get the details
		
        Returns:
            Response containing Definition for Complete profile data
        18.15
        """
        if(account_user_profile_update_model is None):
            raise Exception(self._lr_object.get_validation_message("account_user_profile_update_model"))

        if(self._lr_object.is_null_or_whitespace(uid)):
            raise Exception(self._lr_object.get_validation_message("uid"))

        query_parameters = {}
        query_parameters["apiKey"] = self._lr_object.get_api_key()
        query_parameters["apiSecret"] = self._lr_object.get_api_secret()
        if(not self._lr_object.is_null_or_whitespace(fields)):
            query_parameters["fields"] = fields
        if(null_support is not None):
            query_parameters["nullSupport"] = null_support

        resource_path = "identity/v2/manage/account/" + uid
        return self._lr_object.execute("PUT", resource_path, query_parameters, account_user_profile_update_model)

    def update_phone_id_by_uid(self, phone, uid, fields=''):
        """This API is used to update the PhoneId by using the Uid's. Admin can update the PhoneId's for both the verified and unverified profiles. It will directly replace the PhoneId and bypass the OTP verification process.
        
        Args:
            phone: Phone number
            uid: UID, the unified identifier for each user account
            fields: The fields parameter filters the API response so that the response only includes a specific set of fields
		
        Returns:
            Response containing Definition for Complete profile data
        18.16
        """

        if(self._lr_object.is_null_or_whitespace(phone)):
            raise Exception(self._lr_object.get_validation_message("phone"))

        if(self._lr_object.is_null_or_whitespace(uid)):
            raise Exception(self._lr_object.get_validation_message("uid"))

        query_parameters = {}
        query_parameters["apiKey"] = self._lr_object.get_api_key()
        query_parameters["apiSecret"] = self._lr_object.get_api_secret()
        if(not self._lr_object.is_null_or_whitespace(fields)):
            query_parameters["fields"] = fields

        body_parameters = {}
        body_parameters["phone"] = phone

        resource_path = "identity/v2/manage/account/" + uid + "/phoneid"
        return self._lr_object.execute("PUT", resource_path, query_parameters, body_parameters)

    def get_account_password_hash_by_uid(self, uid):
        """This API use to retrive the hashed password of a specified account in Cloud Storage.
        
        Args:
            uid: UID, the unified identifier for each user account
		
        Returns:
            Response containing Definition for Complete PasswordHash data
        18.17
        """

        if(self._lr_object.is_null_or_whitespace(uid)):
            raise Exception(self._lr_object.get_validation_message("uid"))

        query_parameters = {}
        query_parameters["apiKey"] = self._lr_object.get_api_key()
        query_parameters["apiSecret"] = self._lr_object.get_api_secret()

        resource_path = "identity/v2/manage/account/" + uid + "/password"
        return self._lr_object.execute("GET", resource_path, query_parameters, {})

    def set_account_password_by_uid(self, password, uid):
        """This API is used to set the password of an account in Cloud Storage.
        
        Args:
            password: New password
            uid: UID, the unified identifier for each user account
		
        Returns:
            Response containing Definition for Complete PasswordHash data
        18.18
        """

        if(self._lr_object.is_null_or_whitespace(password)):
            raise Exception(self._lr_object.get_validation_message("password"))

        if(self._lr_object.is_null_or_whitespace(uid)):
            raise Exception(self._lr_object.get_validation_message("uid"))

        query_parameters = {}
        query_parameters["apiKey"] = self._lr_object.get_api_key()
        query_parameters["apiSecret"] = self._lr_object.get_api_secret()

        body_parameters = {}
        body_parameters["password"] = password

        resource_path = "identity/v2/manage/account/" + uid + "/password"
        return self._lr_object.execute("PUT", resource_path, query_parameters, body_parameters)

    def delete_account_by_uid(self, uid):
        """This API deletes the Users account and allows them to re-register for a new account.
        
        Args:
            uid: UID, the unified identifier for each user account
		
        Returns:
            Response containing Definition of Delete Request
        18.19
        """

        if(self._lr_object.is_null_or_whitespace(uid)):
            raise Exception(self._lr_object.get_validation_message("uid"))

        query_parameters = {}
        query_parameters["apiKey"] = self._lr_object.get_api_key()
        query_parameters["apiSecret"] = self._lr_object.get_api_secret()

        resource_path = "identity/v2/manage/account/" + uid
        return self._lr_object.execute("DELETE", resource_path, query_parameters, {})

    def invalidate_account_email_verification(self, uid, email_template='', verification_url=''):
        """This API is used to invalidate the Email Verification status on an account.
        
        Args:
            uid: UID, the unified identifier for each user account
            email_template: Email template name
            verification_url: Email verification url
		
        Returns:
            Response containing Definition of Complete Validation data
        18.20
        """

        if(self._lr_object.is_null_or_whitespace(uid)):
            raise Exception(self._lr_object.get_validation_message("uid"))

        query_parameters = {}
        query_parameters["apiKey"] = self._lr_object.get_api_key()
        query_parameters["apiSecret"] = self._lr_object.get_api_secret()
        if(not self._lr_object.is_null_or_whitespace(email_template)):
            query_parameters["emailTemplate"] = email_template
        if(not self._lr_object.is_null_or_whitespace(verification_url)):
            query_parameters["verificationUrl"] = verification_url

        resource_path = "identity/v2/manage/account/" + uid + "/invalidateemail"
        return self._lr_object.execute("PUT", resource_path, query_parameters, {})

    def get_forgot_password_token(self, email, email_template=None, reset_password_url=None,
        send_email=None):
        """This API Returns a Forgot Password Token it can also be used to send a Forgot Password email to the customer. Note: If you have the UserName workflow enabled, you may replace the 'email' parameter with 'username' in the body.
        
        Args:
            email: user's email
            email_template: Email template name
            reset_password_url: Url to which user should get re-directed to for resetting the password
            send_email: If set to true, the API will also send a Forgot Password email to the customer, bypassing any Bot Protection challenges that they are faced with.
		
        Returns:
            Response containing Definition of Complete Forgot Password data
        18.22
        """

        if(self._lr_object.is_null_or_whitespace(email)):
            raise Exception(self._lr_object.get_validation_message("email"))

        query_parameters = {}
        query_parameters["apiKey"] = self._lr_object.get_api_key()
        query_parameters["apiSecret"] = self._lr_object.get_api_secret()
        if(not self._lr_object.is_null_or_whitespace(email_template)):
            query_parameters["emailTemplate"] = email_template
        if(not self._lr_object.is_null_or_whitespace(reset_password_url)):
            query_parameters["resetPasswordUrl"] = reset_password_url
        if(send_email is not None):
            query_parameters["sendEmail"] = send_email

        body_parameters = {}
        body_parameters["email"] = email

        resource_path = "identity/v2/manage/account/forgot/token"
        return self._lr_object.execute("POST", resource_path, query_parameters, body_parameters)

    def get_email_verification_token(self, email):
        """This API Returns an Email Verification token.
        
        Args:
            email: user's email
		
        Returns:
            Response containing Definition of Complete Verification data
        18.23
        """

        if(self._lr_object.is_null_or_whitespace(email)):
            raise Exception(self._lr_object.get_validation_message("email"))

        query_parameters = {}
        query_parameters["apiKey"] = self._lr_object.get_api_key()
        query_parameters["apiSecret"] = self._lr_object.get_api_secret()

        body_parameters = {}
        body_parameters["email"] = email

        resource_path = "identity/v2/manage/account/verify/token"
        return self._lr_object.execute("POST", resource_path, query_parameters, body_parameters)

    def get_access_token_by_uid(self, uid):
        """The API is used to get LoginRadius access token based on UID.
        
        Args:
            uid: UID, the unified identifier for each user account
		
        Returns:
            Response containing Definition of Complete Token data
        18.24
        """

        if(self._lr_object.is_null_or_whitespace(uid)):
            raise Exception(self._lr_object.get_validation_message("uid"))

        query_parameters = {}
        query_parameters["apiKey"] = self._lr_object.get_api_key()
        query_parameters["apiSecret"] = self._lr_object.get_api_secret()
        query_parameters["uid"] = uid

        resource_path = "identity/v2/manage/account/access_token"
        return self._lr_object.execute("GET", resource_path, query_parameters, {})

    def reset_phone_id_verification_by_uid(self, uid, sms_template=''):
        """This API Allows you to reset the phone no verification of an end user’s account.
        
        Args:
            uid: UID, the unified identifier for each user account
            sms_template: SMS Template name
		
        Returns:
            Response containing Definition of Complete Validation data
        18.27
        """

        if(self._lr_object.is_null_or_whitespace(uid)):
            raise Exception(self._lr_object.get_validation_message("uid"))

        query_parameters = {}
        query_parameters["apiKey"] = self._lr_object.get_api_key()
        query_parameters["apiSecret"] = self._lr_object.get_api_secret()
        if(not self._lr_object.is_null_or_whitespace(sms_template)):
            query_parameters["smsTemplate"] = sms_template

        resource_path = "identity/v2/manage/account/" + uid + "/invalidatephone"
        return self._lr_object.execute("PUT", resource_path, query_parameters, {})

    def upsert_email(self, upsert_email_model, uid, fields=''):
        """This API is used to add/upsert another emails in account profile by different-different email types. If the email type is same then it will simply update the existing email, otherwise it will add a new email in Email array.
        
        Args:
            upsert_email_model: Model Class containing Definition of payload for UpsertEmail Property
            uid: UID, the unified identifier for each user account
            fields: The fields parameter filters the API response so that the response only includes a specific set of fields
		
        Returns:
            Response containing Definition for Complete profile data
        18.29
        """
        if(upsert_email_model is None):
            raise Exception(self._lr_object.get_validation_message("upsert_email_model"))

        if(self._lr_object.is_null_or_whitespace(uid)):
            raise Exception(self._lr_object.get_validation_message("uid"))

        query_parameters = {}
        query_parameters["apiKey"] = self._lr_object.get_api_key()
        query_parameters["apiSecret"] = self._lr_object.get_api_secret()
        if(not self._lr_object.is_null_or_whitespace(fields)):
            query_parameters["fields"] = fields

        resource_path = "identity/v2/manage/account/" + uid + "/email"
        return self._lr_object.execute("PUT", resource_path, query_parameters, upsert_email_model)

    def remove_email(self, email, uid, fields=''):
        """Use this API to Remove emails from a user Account
        
        Args:
            email: user's email
            uid: UID, the unified identifier for each user account
            fields: The fields parameter filters the API response so that the response only includes a specific set of fields
		
        Returns:
            Response containing Definition for Complete profile data
        18.30
        """

        if(self._lr_object.is_null_or_whitespace(email)):
            raise Exception(self._lr_object.get_validation_message("email"))

        if(self._lr_object.is_null_or_whitespace(uid)):
            raise Exception(self._lr_object.get_validation_message("uid"))

        query_parameters = {}
        query_parameters["apiKey"] = self._lr_object.get_api_key()
        query_parameters["apiSecret"] = self._lr_object.get_api_secret()
        if(not self._lr_object.is_null_or_whitespace(fields)):
            query_parameters["fields"] = fields

        body_parameters = {}
        body_parameters["email"] = email

        resource_path = "identity/v2/manage/account/" + uid + "/email"
        return self._lr_object.execute("DELETE", resource_path, query_parameters, body_parameters)

    def refresh_access_token_by_refresh_token(self, refresh__token):
        """This API is used to refresh an access token via it's associated refresh token.
        
        Args:
            refresh__token: LoginRadius refresh token
		
        Returns:
            Response containing Definition of Complete Token data
        18.31
        """

        if(self._lr_object.is_null_or_whitespace(refresh__token)):
            raise Exception(self._lr_object.get_validation_message("refresh__token"))

        query_parameters = {}
        query_parameters["apiKey"] = self._lr_object.get_api_key()
        query_parameters["apiSecret"] = self._lr_object.get_api_secret()
        query_parameters["refresh_Token"] = refresh__token

        resource_path = "identity/v2/manage/account/access_token/refresh"
        return self._lr_object.execute("GET", resource_path, query_parameters, {})

    def revoke_refresh_token(self, refresh__token):
        """The Revoke Refresh Access Token API is used to revoke a refresh token or the Provider Access Token, revoking an existing refresh token will invalidate the refresh token but the associated access token will work until the expiry.
        
        Args:
            refresh__token: LoginRadius refresh token
		
        Returns:
            Response containing Definition of Delete Request
        18.32
        """

        if(self._lr_object.is_null_or_whitespace(refresh__token)):
            raise Exception(self._lr_object.get_validation_message("refresh__token"))

        query_parameters = {}
        query_parameters["apiKey"] = self._lr_object.get_api_key()
        query_parameters["apiSecret"] = self._lr_object.get_api_secret()
        query_parameters["refresh_Token"] = refresh__token

        resource_path = "identity/v2/manage/account/access_token/refresh/revoke"
        return self._lr_object.execute("GET", resource_path, query_parameters, {})

    def get_account_identities_by_email(self, email, fields=''):
        """Note: This is intended for specific workflows where an email may be associated to multiple UIDs. This API is used to retrieve all of the identities (UID and Profiles), associated with a specified email in Cloud Storage.
        
        Args:
            email: Email of the user
            fields: The fields parameter filters the API response so that the response only includes a specific set of fields
		
        Returns:
            Complete user Identity data
        18.35
        """

        if(self._lr_object.is_null_or_whitespace(email)):
            raise Exception(self._lr_object.get_validation_message("email"))

        query_parameters = {}
        query_parameters["apiKey"] = self._lr_object.get_api_key()
        query_parameters["apiSecret"] = self._lr_object.get_api_secret()
        query_parameters["email"] = email
        if(not self._lr_object.is_null_or_whitespace(fields)):
            query_parameters["fields"] = fields

        resource_path = "identity/v2/manage/account/identities"
        return self._lr_object.execute("GET", resource_path, query_parameters, {})

    def account_delete_by_email(self, email):
        """This API is used to delete all user profiles associated with an Email.
        
        Args:
            email: Email of the user
		
        Returns:
            Response containing Definition of Delete Request
        18.36
        """

        if(self._lr_object.is_null_or_whitespace(email)):
            raise Exception(self._lr_object.get_validation_message("email"))

        query_parameters = {}
        query_parameters["apiKey"] = self._lr_object.get_api_key()
        query_parameters["apiSecret"] = self._lr_object.get_api_secret()
        query_parameters["email"] = email

        resource_path = "identity/v2/manage/account"
        return self._lr_object.execute("DELETE", resource_path, query_parameters, {})

    def account_update_uid(self, update_uid_model, uid):
        """This API is used to update a user's Uid. It will update all profiles, custom objects and consent management logs associated with the Uid.
        
        Args:
            update_uid_model: Payload containing Update UID
            uid: UID, the unified identifier for each user account
		
        Returns:
            Response containing Definition of Complete Validation data
        18.41
        """
        if(update_uid_model is None):
            raise Exception(self._lr_object.get_validation_message("update_uid_model"))

        if(self._lr_object.is_null_or_whitespace(uid)):
            raise Exception(self._lr_object.get_validation_message("uid"))

        query_parameters = {}
        query_parameters["apiKey"] = self._lr_object.get_api_key()
        query_parameters["apiSecret"] = self._lr_object.get_api_secret()
        query_parameters["uid"] = uid

        resource_path = "identity/v2/manage/account/uid"
        return self._lr_object.execute("PUT", resource_path, query_parameters, update_uid_model)
